      program oab
c
c     main calling program for the 2 level CRD 
c     with background continuum problem of 
c     Olson, Auer Buchler 1986 JQSRT 35, 431
c
      include 'prec'
      parameter (ndep=70, nfrq=30, nmu=3)
      dimension tau(ndep), s(ndep), epsilon(ndep), b(ndep),
     * pe(ndep), s1(ndep), starl(ndep),jbar(ndep),
     * s2(ndep),s3(ndep),s4(ndep),sng(ndep),wt(ndep),t(ndep)
      dimension p(ndep), pms(ndep), iplus(0:ndep),iminus(0:ndep)
      real*8 iplus,iminus,jbar
      dimension x(0:nfrq), phi(0:nfrq), wphi(0:nfrq)
      dimension xmu(nmu), wmu(nmu)
      logical ngon
      ngon=.true.
C      ngon=.false.
      iout=1
      crit=1.e-10
      itmax=400
      itran=0
c
c depth grid, constant d log tau is used.
c      
      taumin=0.001
      taumax=1.e8
      dlgtau=log10(taumax/taumin)/float(ndep-1)
      dtau=10.**dlgtau
      tau(1)=taumin
      do 100 i=2,ndep
         tau(i)=tau(i-1)*dtau
 100  continue
c
c  epsilon and B parameters
c      
      if(iout .eq. 1) write(6,*)' I  Tau        Epsilon    B'
      do 101 i=1,ndep
         epsilon(i)=1.e-4
         b(i)=1.
c         if(tau(i) .gt. 1. .and. tau(i) .lt. 100.) b(i)= 3.
         if(iout .eq. 1) write(6,1000) i, tau(i),epsilon(i),b(i)
 101  continue
 1000 format(i3, 3(1x,1p,e10.3))
c
c  line profile, assumed the same at each depth
c  trapezoidal weights, multiplied by two for a one-sided profile
c
      ndw=4
      ptot=0.
      dx=1./float(nfrq)*float(ndw)
      do 102 i=0, nfrq
         x(i)=float(i)*dx
         phi(i)=exp(-(x(i)*x(i)))
         wphi(i)= 2.0*dx
         if(i .eq. 0 .or. i .eq. nfrq) wphi(i)=wphi(i)/2.
         ptot=ptot+phi(i)*wphi(i)
 102  continue
c
c Normalize so integral is unity 
c
      do 1021 i=0,nfrq
         phi(i)=phi(i)/ptot
 1021 continue
      tmin=phi(nfrq)*tau(ndep)
      if(iout .eq. 1) then 
         write(6,*)' I  X          Phi        Wphi'
         do 104 i=0, nfrq
            write(6,1000) i, x(i),phi(i), wphi(i)
 104     continue
      endif
      if(tmin .lt. 1.0) then
         write(6,*) 'Atmosphere is too thin: tau max =', tmin
         stop
      else 
         write(6,*) 'Base optical depth in wing =',tmin
      endif
c
c  Gaussian quadrature in angle
c
      x1=0.
      x2=1.
      call gausi(3,x1,x2,wmu,xmu)
      if(iout .eq. 1) write(6,*)' I  Mu           Xmu'
      do 103 i=1, nmu
         if(iout .eq. 1) write(6,1001) i, xmu(i),wmu(i)
 103  continue
 1001 format(i3, 2(1x,1p,e10.3))
c
c  Starting approximation, eq (16) of OAB, AND
c  computation of Lambda* (variable starl), eqs 19 and 20 of OAB
c
      if(iout .eq. 1) write(6,*)
     * ' I  tau        p          1/(1+tau)   S(start)    S(start)/B
     * Lambda*'  
      do 107 k=1,ndep
         pe(k)=0.
         starl(k)=0.
         do 105 i=0,nfrq
            uistar=0.
            do 106 j=1,nmu
               pe(k)=pe(k)+wmu(j)*wphi(i)*phi(i)*
     *              exp(-phi(i)*tau(k)/xmu(j))
c
c  construct lambda star:
c  this is eq 31 of Hubeny 1992 (Feautrier form)
c
c  include approximate boundary values for lambda* (k =1, k=ndep)
c
               if(k .eq. 1) then 
                  dtdm1=(tau(k)-0.)*phi(i)/xmu(j)
               else
                  dtdm1=(tau(k)-tau(k-1))*phi(i)/xmu(j)
               endif
               if(k .eq. ndep) then 
                  dtd=dtdm1
               else
                  dtd=(tau(k+1)-tau(k))*phi(i)/xmu(j)
               endif
               dt=(dtd+dtdm1)/2.
               t1=(1. - exp(-dtd))/dtd
               t2=(1. - exp(-dtdm1))/dtdm1
C OAB BEGIN  **************************************************
               uistar= 1. / (1. + (t1/dt + t2/dt))
               starl(k)=starl(k) + uistar*phi(i)*wphi(i)*wmu(j)
 106        continue
C OAB END    **************************************************
C OK BEGIN   **************************************************
C               uistar = uistar + (t1+t2)*wmu(j)
C 106        continue
C            starl(k) = starl(k) +(1.-0.5*uistar)*phi(i)*wphi(i)
C OK END     **************************************************
 105     continue
c eq (16)
c         s1(k)=epsilon(k)*b(k)/ (1. - (1.-epsilon(k))*(1.-pe(k)))
c eq (29) of Scharmer 1981
         s1(k)=b(k)*sqrt(epsilon(k)/ (epsilon(k)+2.*pe(k)))
c         s1(k)=b(k)
         s(k)=s1(k)
         if(iout .eq. 1) write(6,1002) k, tau(k),pe(k),
     *   1./(1.+tau(k)),s1(k),s1(k)/b(k), starl(k)
 107  continue
 1002 format(i3, 6(1x,1p,e10.3))
c
c  main loop:
c  Now compute RHS of 22, get formal solution and update source fn.
c      
c      goto 998
      kount=0
 999  continue
      kount=kount+1
      dsmax=0.
      do 112 k=1,ndep
         jbar(k)=0.
 112  continue
c solve transfer problem and compute jbar:
      do 110 j=1,nmu
         xmy=xmu(j)
c
c  If incoming radiation use itran ge 10
c
c         iminus(0)=2.
c         itran=10
         iminus(0)=0.
         itran=0
         do 111 i=0,nfrq
C begin June 14, 2000
            do 1131 k=1,ndep
               t(k)=tau(k)*phi(i)
 1131       continue
            call transp(xmy,ndep,t,s,itran,p,pms,iplus,iminus)
C           call transp(xmy,ndep,tau,s,itran,p,pms,iplus,iminus)
C end June 14, 2000
            do 113 k=1,ndep
               jbar(k)=jbar(k)+p(k)*phi(i)*wphi(i)*wmu(j)
 113        continue
 111     continue
 110  continue
c      if(iout .eq. 1) write(6,*)
c     * ' I sold       jbar    starl*sold     snew'
      do 114 k=1,ndep
         s4(k)=s3(k)
         s3(k)=s2(k)
         s2(k)=s1(k)
         rhs= (1.-epsilon(k))* (jbar(k) -starl(k)*s1(k)) + 
     *        epsilon(k)*b(k)
         s(k)=rhs / (1. - (1.-epsilon(k))*starl(k))
         ds=(s(k)-s1(k))/s1(k)
c         if(iout .eq. 1) write(6,1002) k, s1(k),jbar(k), 
c     *                   starl(k)*s1(k),s(k)
         s1(k)=s(k)
         wt(k)=1./jbar(k)
         dsmax = max(dsmax,abs(ds))
 114  continue
      write(6,*) 'Iteration ',kount,' max relative change in s is',dsmax
      if (ngon .and. kount .ge. 4 .and. mod(kount,3) .eq. 0) then 
         write(6,*)'NG acceleration'
         call ng(ndep,wt,s,s2,s3,s4)
         do 117 k=1,ndep
            s(k)=s4(k)
            s1(k)=s(k)
 117     continue
      endif
      if(kount .ge. itmax) stop
      if(dsmax .gt. crit) goto 999
      write(6,*) 'SOLUTION:'
      write(6,*) 'depth tau        B          S '
      do 115 k=1,ndep-1
         write(*,1002)k,tau(k), b(k), s(k)
 115  continue
 998  continue
      stop
      end
